package com.microsoft.flutterappcenter;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface; 
import android.net.Uri;

import com.microsoft.appcenter.AppCenter;
import com.microsoft.appcenter.analytics.Analytics;
import com.microsoft.appcenter.crashes.Crashes;
import com.microsoft.appcenter.distribute.Distribute;
import com.microsoft.appcenter.distribute.UpdateTrack;

import com.microsoft.appcenter.distribute.DistributeListener; 
import com.microsoft.appcenter.distribute.ReleaseDetails; 
import com.microsoft.appcenter.distribute.UpdateAction;

import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterBuglyPlugin
 */
public class FlutterAppcenterPlugin implements MethodCallHandler,DistributeListener {
    private Activity activity;
    private Result result;

    private String updateDialogTitle;
    private String updateDialogSubTitle;
    private String updateDialogDetail;
    private String updateDialogConfirm;
    private String updateDialogCancel;

    public FlutterAppcenterPlugin(Activity activity) {
        this.activity = activity;
        this.updateDialogTitle = "Update";
        this.updateDialogSubTitle = "";
        this.updateDialogDetail = "";
        this.updateDialogConfirm = "Confirm";
        this.updateDialogCancel = "Cancel";
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_appcenter");
        FlutterAppcenterPlugin plugin = new FlutterAppcenterPlugin(registrar.activity());
        channel.setMethodCallHandler(plugin);

        Distribute.setListener(plugin);
    }

    @Override
    public void onMethodCall(final MethodCall call, final Result result) {
        this.result = result;
        if(call.method.equals("initAppCenter")) {
            if(call.hasArgument("usePrivateTrack")) {
                String usePrivateTrackStr = call.argument("usePrivateTrack").toString();
                boolean usePrivateTrack = Boolean.parseBoolean(usePrivateTrackStr);
                System.out.println("usePrivateTrack:" + usePrivateTrack);

                if(usePrivateTrack){
                    System.out.println("usePrivateTrack:ppp");
                    Distribute.setUpdateTrack(UpdateTrack.PRIVATE);
                }
            }

            if(call.hasArgument("automaticCheckForUpdate")) {
                String automaticCheckForUpdateStr = call.argument("automaticCheckForUpdate").toString();
                boolean automaticCheckForUpdate = Boolean.parseBoolean(automaticCheckForUpdateStr);
                System.out.println("automaticCheckForUpdate:" + automaticCheckForUpdate);

                if(!automaticCheckForUpdate){
                    Distribute.disableAutomaticCheckForUpdate();
                }
            }

            if(call.hasArgument("updateDialogTitle")){
                this.updateDialogTitle = call.argument("updateDialogTitle").toString();
            }

            if(call.hasArgument("updateDialogSubTitle")){
                this.updateDialogSubTitle = call.argument("updateDialogSubTitle").toString();
            }

            if(call.hasArgument("updateDialogDetail")){
                this.updateDialogDetail = call.argument("updateDialogDetail").toString();
            }

            if(call.hasArgument("updateDialogConfirm")){
                this.updateDialogConfirm = call.argument("updateDialogConfirm").toString();
            }

            if(call.hasArgument("updateDialogCancel")){
                this.updateDialogCancel = call.argument("updateDialogCancel").toString();
            }

            if(call.hasArgument("appSecret")) {
                String appSecret = call.argument("appSecret").toString();

                AppCenter.start(activity.getApplication(), appSecret, Analytics.class, Crashes.class, Distribute.class);
                result.success("Initialize Successfully");
            }else {
                result.error("UNAVAILABLE", "AppCenter appSecret is null", null);
            }
        }else if(call.method.equals("checkForUpdate")){
            Distribute.checkForUpdate();
        }else if(call.method.equals("isEnabledForDistribute")){
            boolean isEnabled = Distribute.isEnabled().get();
            System.out.println("isEnabledForDistribute:" + isEnabled);

            String res = isEnabled ? "1" : "0";
            result.success(res);
        }else{
            result.error("UNAVAILABLE", "it can not implement initAppCenter method", null);
        }

    }

    @Override
    public boolean onReleaseAvailable(Activity activity, ReleaseDetails releaseDetails) {
        // Look at releaseDetails public methods to get version information, release notes text or release notes URL
        String versionName = releaseDetails.getShortVersion();
        int versionCode = releaseDetails.getVersion();
        String releaseNotes = releaseDetails.getReleaseNotes();
        Uri releaseNotesUrl = releaseDetails.getReleaseNotesUrl();

        if(this.updateDialogSubTitle.length() != 0){
            this.updateDialogSubTitle = this.updateDialogSubTitle + ":"; 
        }
    
        if(this.updateDialogDetail.length() == 0){
            this.updateDialogDetail = releaseNotes; 
        }
    
        this.updateDialogDetail = this.updateDialogSubTitle + versionName + "(" + String.valueOf(versionCode) + ")\n" + this.updateDialogDetail;

        // Build our own dialog title and message
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(activity);
        dialogBuilder.setTitle(this.updateDialogTitle); // you should use a string resource instead of course, this is just to simplify example
        dialogBuilder.setMessage(this.updateDialogDetail);

        // Mimic default SDK buttons
        dialogBuilder.setPositiveButton(this.updateDialogConfirm, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {

                // This method is used to tell the SDK what button was clicked
                Distribute.notifyUpdateAction(UpdateAction.UPDATE);
            }
        });

        // We can postpone the release only if the update is not mandatory
        if (!releaseDetails.isMandatoryUpdate()) {
            dialogBuilder.setNegativeButton(this.updateDialogCancel, new DialogInterface.OnClickListener() {

                @Override
                public void onClick(DialogInterface dialog, int which) {

                    // This method is used to tell the SDK what button was clicked
                    Distribute.notifyUpdateAction(UpdateAction.POSTPONE);
                }
            });
        }
        dialogBuilder.setCancelable(false); // if it's cancelable you should map cancel to postpone, but only for optional updates
        dialogBuilder.create().show();

        // Return true if you are using your own dialog, false otherwise
        return true;
    }
}