#import "FlutterAppcenterPlugin.h"

@import AppCenter;
@import AppCenterAnalytics;
@import AppCenterCrashes;
@import AppCenterDistribute;

@interface FlutterAppcenterPlugin () <MSDistributeDelegate>

@property(nonatomic) NSString *updateDialogTitle;
@property(nonatomic) NSString *updateDialogSubTitle;
@property(nonatomic) NSString *updateDialogDetail;
@property(nonatomic) NSString *updateDialogConfirm;
@property(nonatomic) NSString *updateDialogCancel;

@end

@implementation FlutterAppcenterPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel =
      [FlutterMethodChannel methodChannelWithName:@"flutter_appcenter"
                                  binaryMessenger:[registrar messenger]];

  FlutterAppcenterPlugin* instance = [[FlutterAppcenterPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];

  [MSDistribute setDelegate:instance];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if([@"initAppCenter" isEqualToString:call.method]) {

    NSString *appSecret = call.arguments[@"appSecret"];
    NSLog(@"flutter_appcenter appSecret: %@", appSecret);
    
    NSString *usePrivateTrack = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"usePrivateTrack"]];
    if([@"1" isEqualToString:usePrivateTrack]) {
      MSDistribute.updateTrack = MSUpdateTrackPrivate;
      NSLog(@"updateTrack: MSUpdateTrackPrivate");
    }

    NSString *automaticCheckForUpdate = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"automaticCheckForUpdate"]];
    if([@"0" isEqualToString:automaticCheckForUpdate]){
      [MSDistribute disableAutomaticCheckForUpdate];
      NSLog(@"automaticCheckForUpdate: disable");
    }

    self.updateDialogTitle = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"updateDialogTitle"]];
    self.updateDialogSubTitle = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"updateDialogSubTitle"]];
    self.updateDialogDetail = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"updateDialogDetail"]];
    self.updateDialogConfirm = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"updateDialogConfirm"]];
    self.updateDialogCancel = [[NSString alloc] initWithFormat:@"%@",call.arguments[@"updateDialogCancel"]];

    BOOL validResult = [self initAppCenter:appSecret];
    if(validResult) {
      result(@("Initialize Successfully"));
    }else {
      result(@("appSecret is null"));
    }

  }else if([@"checkForUpdate" isEqualToString:call.method]) {
    [MSDistribute checkForUpdate];
  }else if([@"isEnabledForDistribute" isEqualToString:call.method]) {
    [MSDistribute isEnabled] ? result(@("1")) : result(@("0"));
  }else {
    result(FlutterMethodNotImplemented);
  }

}

- (BOOL)initAppCenter:(NSString *)appSecret {
  BOOL valid = [self isNotEmpty:appSecret];

  if(valid){
    // #if DEBUG
    //   [MSAppCenter start:appSecret withServices:@[
    //     [MSAnalytics class],
    //     [MSCrashes class]
    //   ]];
    // #else
      [MSAppCenter start:appSecret withServices:@[
        [MSAnalytics class],
        [MSCrashes class],
        [MSDistribute class]
      ]];
    // #endif
  }

  return valid;
}

- (BOOL) isNotEmpty:(NSString *)string {
  if(string == nil || string == NULL) {
    return NO;
  }
  
  if([string isKindOfClass:[NSNull class]]) {
    return NO;
  }
  if([[string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] length] == 0) {
    return NO;
  }

  return YES;
}

- (BOOL)distribute:(MSDistribute *)distribute releaseAvailableWithDetails:(MSReleaseDetails *)details {

  if(![self.updateDialogSubTitle isEqualToString:@""]){
    self.updateDialogSubTitle = [[NSString alloc] initWithFormat:@"%@:", self.updateDialogSubTitle];
  }

  if([self.updateDialogDetail isEqualToString:@""]){
    self.updateDialogDetail = details.releaseNotes;
  }

  self.updateDialogDetail = [[NSString alloc] initWithFormat:@"%@ %@(%@)\n%@", self.updateDialogSubTitle, details.shortVersion, details.version, self.updateDialogDetail];

  UIAlertController *alertController = [UIAlertController
                                          alertControllerWithTitle: self.updateDialogTitle
                                          message: self.updateDialogDetail
                                          preferredStyle: UIAlertControllerStyleAlert];

  if(!details.mandatoryUpdate){
      [alertController addAction:[UIAlertAction
                                      actionWithTitle: self.updateDialogCancel
                                      style: UIAlertActionStyleDefault
                                      handler: ^(UIAlertAction *action) {
                                        [MSDistribute notifyUpdateAction:MSUpdateActionPostpone];
                                      }
                                    ]];
  }

  [alertController addAction:[UIAlertAction
                                actionWithTitle: self.updateDialogConfirm
                                style: UIAlertActionStyleCancel
                                handler: ^(UIAlertAction *action) {
                                  [MSDistribute notifyUpdateAction: MSUpdateActionUpdate];
                                }
                              ]];

  self.window = [[UIApplication sharedApplication].windows lastObject];
  [self.window.rootViewController presentViewController:alertController animated:YES completion:nil];
  return YES;
}
@end
