library flutter_appcenter;

import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

class FlutterAppCenter {
  static const MethodChannel _channel =
      const MethodChannel('flutter_appcenter');

  /// start running the AppCenert
  static Future<String> init({
    @required String appSecretAndroid,
    @required String appSecretIOS,

    /// set private or public distribution group
    /// defualut value is publice
    bool usePrivateTrack = false,

    /// when automaticCheckForUpdate is false, can use checkForUpdate method to update
    bool automaticCheckForUpdate = true,

    /// Customize or localize the in-app update dialog
    Map<String,String> updateDialog
  }) async {
    Map<String, Object> args = {
      'appSecret': Platform.isAndroid ? appSecretAndroid : appSecretIOS,
      'usePrivateTrack': usePrivateTrack,
      'automaticCheckForUpdate': automaticCheckForUpdate,
    };

    if(updateDialog == null){
      args['updateDialogTitle'] = 'App update avaiable';
      args['updateDialogSubTitle'] = '';
      args['updateDialogDetail'] = ''; // defualt to show the releaseNotes and version
      args['updateDialogConfirm'] = 'Confirm';
      args['updateDialogCancel'] = 'Postpone';
    }else{
      args['updateDialogTitle'] = updateDialog.containsKey('title') ? updateDialog['title'] : 'App update avaiable';
      args['updateDialogSubTitle'] = updateDialog.containsKey('subTitle') ? updateDialog['subTitle'] : '';
      args['updateDialogDetail'] = updateDialog.containsKey('detail') ? updateDialog['detail'] : ''; // defualt to show the releaseNotes and version
      args['updateDialogConfirm'] = updateDialog.containsKey('confirm') ? updateDialog['confirm'] : 'Confirm';
      args['updateDialogCancel'] = updateDialog.containsKey('cancel') ? updateDialog['cancel'] : 'Postpone';
    }

    try {
      return await _channel.invokeMethod('initAppCenter', args);
    } on PlatformException catch (e) {
      print("Failed to start running the AppCenert: '${e.message}'.");
    }
  }

  /// when automaticCheckForUpdate is false that means disable to update automatically,
  /// need to Check Manually for Update
  static Future<void> checkForUpdate() async{
    try {
      await _channel.invokeMethod('checkForUpdate');
    } on PlatformException catch (e) {
      print("Failed to check for Update: '${e.message}'.");
    }
  }

  //Check if App Center Distribute is enabled
  static Future<bool> isEnabledForDistribute() async{
    try {
      String result = await _channel.invokeMethod('isEnabledForDistribute');
      return result == '1';
    } on PlatformException catch (e) {
      print("Failed to check for Distribute: '${e.message}'.");
    }
  }

}
